package gamma;

import PrologDB.TableSchema;
import PrologDB.Tuple;
import gammaSupport.BMap;
import gammaSupport.Connector;
import gammaSupport.ReadEnd;
import gammaSupport.ThreadList;
import gammaSupport.WriteEnd;

/** primitive HSplit */
public class HSplit extends Thread implements gammaSupport.GammaConstants {

    private final String joinkey;
    private final ReadEnd in;
    private final WriteEnd[] out;

    /**
     * 
     * @param in -- input connector
     * @param out -- output connector
     * @param hashkey on which hashing is to be done, a field of input connector
     */
    HSplit(Connector in, Connector[] out, String hashkey) {
        int i;

        this.joinkey = hashkey;
        this.in = in.getReadEnd();
        this.out = new WriteEnd[splitLen];
        for (i = 0; i < splitLen; i++) {
            this.out[i] = out[i].getWriteEnd();
        }
        TableSchema ts = in.getTableSchema();
        for (i = 0; i < splitLen; i++) {
            out[i].setTableSchema(ts);
        }
        
        in.verifyField(hashkey, "HSplit");
        ThreadList.add(this);
    }

    /** body of HSplit */
    @Override
    public void run() {
        Tuple t;
        int hash, i, j;

        while (true) {
            t = in.getNextTuple();
            if (t == null) {
                for (i = 0; i < splitLen; i++) {
                    out[i].close();
                }
                return;
            }
            hash = BMap.myhash(t.get(joinkey));
            out[hash].putNextTuple(t);
        }
    }
}
