package gamma;

import PrologDB.TableSchema;
import PrologDB.Tuple;
import gammaSupport.ReadEnd;
import gammaSupport.WriteEnd;
import gammaSupport.Connector;
import gammaSupport.GammaDebug;
import static gammaSupport.GammaDebug.streamOutput;
import gammaSupport.GammaError;
import static gammaSupport.GammaError.tableSchemasNoMatch;
import gammaSupport.ThreadList;

/** merge box */
public class Merge extends Thread implements gammaSupport.GammaConstants {

    ReadEnd[] in;
    WriteEnd out;
    boolean[] done;
    int numberDone = 0;

    /**
     * merge box constructor
     * @param in -- input array of connectors
     * @param out -- output connector
     */
    Merge(Connector in[], Connector out) {

        this.in = new ReadEnd[splitLen];
        done = new boolean[splitLen];
        for (int i = 0; i < splitLen; i++) {
            this.in[i] = in[i].getReadEnd();
            done[i] = false;
        }

        this.out = out.getWriteEnd();
        TableSchema ts = in[0].getTableSchema();
        for (int i = 1; i < splitLen; i++) {
            if (in[i] != null && !ts.equals(in[i].getTableSchema())) {
                throw GammaError.toss(tableSchemasNoMatch, in[0].getName());
            }
        }
        out.setTableSchema(ts);
        
        ThreadList.add(this);
    }

    /** Merge body */
    @Override
    public void run() {
        while (numberDone != splitLen) {
            for (int i = 0; i < splitLen; i++) {
                if (!done[i]) {
                    //if (GammaUtil.debug) System.err.println("waiting to read stream " +i);

                    Tuple t = in[i].getNextTuple();
                    if (t == null) {
                        done[i] = true;
                        numberDone++;
                        in[i].close();
                        continue;
                    }
                    out.putNextTuple(t);
                    GammaDebug.report(streamOutput, i + "", t.toString());
                }
            }
        }
        out.close();
    }
}
